=begin
#Custom Workflow Actions

#Create custom workflow actions

The version of the OpenAPI document: v4

Generated by: https://openapi-generator.tech
OpenAPI Generator version: 4.3.1

=end

require 'cgi'

module Hubspot
  module Automation
    module Actions
      class DefinitionsApi
        attr_accessor :api_client

        def initialize(api_client = ApiClient.default)
          @api_client = api_client
        end
        # Archive a custom action
        # Archives a single custom workflow action with the specified ID. Workflows that currently use this custom action will stop attempting to execute the action, and all future executions will be marked as a failure.
        # @param definition_id [String] The ID of the custom workflow action.
        # @param app_id [Integer] 
        # @param [Hash] opts the optional parameters
        # @return [nil]
        def archive(definition_id, app_id, opts = {})
          archive_with_http_info(definition_id, app_id, opts)
          nil
        end

        # Archive a custom action
        # Archives a single custom workflow action with the specified ID. Workflows that currently use this custom action will stop attempting to execute the action, and all future executions will be marked as a failure.
        # @param definition_id [String] The ID of the custom workflow action.
        # @param app_id [Integer] 
        # @param [Hash] opts the optional parameters
        # @return [Array<(nil, Integer, Hash)>] nil, response status code and response headers
        def archive_with_http_info(definition_id, app_id, opts = {})
          if @api_client.config.debugging
            @api_client.config.logger.debug 'Calling API: DefinitionsApi.archive ...'
          end
          # verify the required parameter 'definition_id' is set
          if @api_client.config.client_side_validation && definition_id.nil?
            fail ArgumentError, "Missing the required parameter 'definition_id' when calling DefinitionsApi.archive"
          end
          # verify the required parameter 'app_id' is set
          if @api_client.config.client_side_validation && app_id.nil?
            fail ArgumentError, "Missing the required parameter 'app_id' when calling DefinitionsApi.archive"
          end
          # resource path
          local_var_path = '/automation/v4/actions/{appId}/{definitionId}'.sub('{' + 'definitionId' + '}', CGI.escape(definition_id.to_s)).sub('{' + 'appId' + '}', CGI.escape(app_id.to_s))

          # query parameters
          query_params = opts[:query_params] || {}

          # header parameters
          header_params = opts[:header_params] || {}
          # HTTP header 'Accept' (if needed)
          header_params['Accept'] = @api_client.select_header_accept(['*/*'])

          # form parameters
          form_params = opts[:form_params] || {}

          # http body (model)
          post_body = opts[:body] 

          # return_type
          return_type = opts[:return_type] 

          # auth_names
          auth_names = opts[:auth_names] || ['developer_hapikey']

          new_options = opts.merge(
            :header_params => header_params,
            :query_params => query_params,
            :form_params => form_params,
            :body => post_body,
            :auth_names => auth_names,
            :return_type => return_type
          )

          data, status_code, headers = @api_client.call_api(:DELETE, local_var_path, new_options)
          if @api_client.config.debugging
            @api_client.config.logger.debug "API called: DefinitionsApi#archive\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
          end
          return data, status_code, headers
        end

        # Create new custom action
        # Creates a new custom workflow action.
        # @param app_id [Integer] 
        # @param extension_action_definition_input [ExtensionActionDefinitionInput] The custom workflow action to create.
        # @param [Hash] opts the optional parameters
        # @return [ExtensionActionDefinition]
        def create(app_id, extension_action_definition_input, opts = {})
          data, _status_code, _headers = create_with_http_info(app_id, extension_action_definition_input, opts)
          data
        end

        # Create new custom action
        # Creates a new custom workflow action.
        # @param app_id [Integer] 
        # @param extension_action_definition_input [ExtensionActionDefinitionInput] The custom workflow action to create.
        # @param [Hash] opts the optional parameters
        # @return [Array<(ExtensionActionDefinition, Integer, Hash)>] ExtensionActionDefinition data, response status code and response headers
        def create_with_http_info(app_id, extension_action_definition_input, opts = {})
          if @api_client.config.debugging
            @api_client.config.logger.debug 'Calling API: DefinitionsApi.create ...'
          end
          # verify the required parameter 'app_id' is set
          if @api_client.config.client_side_validation && app_id.nil?
            fail ArgumentError, "Missing the required parameter 'app_id' when calling DefinitionsApi.create"
          end
          # verify the required parameter 'extension_action_definition_input' is set
          if @api_client.config.client_side_validation && extension_action_definition_input.nil?
            fail ArgumentError, "Missing the required parameter 'extension_action_definition_input' when calling DefinitionsApi.create"
          end
          # resource path
          local_var_path = '/automation/v4/actions/{appId}'.sub('{' + 'appId' + '}', CGI.escape(app_id.to_s))

          # query parameters
          query_params = opts[:query_params] || {}

          # header parameters
          header_params = opts[:header_params] || {}
          # HTTP header 'Accept' (if needed)
          header_params['Accept'] = @api_client.select_header_accept(['application/json', '*/*'])
          # HTTP header 'Content-Type'
          header_params['Content-Type'] = @api_client.select_header_content_type(['application/json'])

          # form parameters
          form_params = opts[:form_params] || {}

          # http body (model)
          post_body = opts[:body] || @api_client.object_to_http_body(extension_action_definition_input) 

          # return_type
          return_type = opts[:return_type] || 'ExtensionActionDefinition' 

          # auth_names
          auth_names = opts[:auth_names] || ['developer_hapikey']

          new_options = opts.merge(
            :header_params => header_params,
            :query_params => query_params,
            :form_params => form_params,
            :body => post_body,
            :auth_names => auth_names,
            :return_type => return_type
          )

          data, status_code, headers = @api_client.call_api(:POST, local_var_path, new_options)
          if @api_client.config.debugging
            @api_client.config.logger.debug "API called: DefinitionsApi#create\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
          end
          return data, status_code, headers
        end

        # Get a custom action
        # Returns a single custom workflow action with the specified ID.
        # @param definition_id [String] The ID of the custom workflow action.
        # @param app_id [Integer] 
        # @param [Hash] opts the optional parameters
        # @option opts [Boolean] :archived Whether to include archived custom actions. (default to false)
        # @return [ExtensionActionDefinition]
        def get_by_id(definition_id, app_id, opts = {})
          data, _status_code, _headers = get_by_id_with_http_info(definition_id, app_id, opts)
          data
        end

        # Get a custom action
        # Returns a single custom workflow action with the specified ID.
        # @param definition_id [String] The ID of the custom workflow action.
        # @param app_id [Integer] 
        # @param [Hash] opts the optional parameters
        # @option opts [Boolean] :archived Whether to include archived custom actions.
        # @return [Array<(ExtensionActionDefinition, Integer, Hash)>] ExtensionActionDefinition data, response status code and response headers
        def get_by_id_with_http_info(definition_id, app_id, opts = {})
          if @api_client.config.debugging
            @api_client.config.logger.debug 'Calling API: DefinitionsApi.get_by_id ...'
          end
          # verify the required parameter 'definition_id' is set
          if @api_client.config.client_side_validation && definition_id.nil?
            fail ArgumentError, "Missing the required parameter 'definition_id' when calling DefinitionsApi.get_by_id"
          end
          # verify the required parameter 'app_id' is set
          if @api_client.config.client_side_validation && app_id.nil?
            fail ArgumentError, "Missing the required parameter 'app_id' when calling DefinitionsApi.get_by_id"
          end
          # resource path
          local_var_path = '/automation/v4/actions/{appId}/{definitionId}'.sub('{' + 'definitionId' + '}', CGI.escape(definition_id.to_s)).sub('{' + 'appId' + '}', CGI.escape(app_id.to_s))

          # query parameters
          query_params = opts[:query_params] || {}
          query_params[:'archived'] = opts[:'archived'] if !opts[:'archived'].nil?

          # header parameters
          header_params = opts[:header_params] || {}
          # HTTP header 'Accept' (if needed)
          header_params['Accept'] = @api_client.select_header_accept(['application/json', '*/*'])

          # form parameters
          form_params = opts[:form_params] || {}

          # http body (model)
          post_body = opts[:body] 

          # return_type
          return_type = opts[:return_type] || 'ExtensionActionDefinition' 

          # auth_names
          auth_names = opts[:auth_names] || ['developer_hapikey']

          new_options = opts.merge(
            :header_params => header_params,
            :query_params => query_params,
            :form_params => form_params,
            :body => post_body,
            :auth_names => auth_names,
            :return_type => return_type
          )

          data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
          if @api_client.config.debugging
            @api_client.config.logger.debug "API called: DefinitionsApi#get_by_id\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
          end
          return data, status_code, headers
        end

        # Get all custom actions
        # Returns a list of all custom workflow actions.
        # @param app_id [Integer] 
        # @param [Hash] opts the optional parameters
        # @option opts [Integer] :limit Maximum number of results per page.
        # @option opts [String] :after The paging cursor token of the last successfully read resource will be returned as the &#x60;paging.next.after&#x60; JSON property of a paged response containing more results.
        # @option opts [Boolean] :archived Whether to include archived custom actions. (default to false)
        # @return [CollectionResponseExtensionActionDefinitionForwardPaging]
        def get_page(app_id, opts = {})
          data, _status_code, _headers = get_page_with_http_info(app_id, opts)
          data
        end

        # Get all custom actions
        # Returns a list of all custom workflow actions.
        # @param app_id [Integer] 
        # @param [Hash] opts the optional parameters
        # @option opts [Integer] :limit Maximum number of results per page.
        # @option opts [String] :after The paging cursor token of the last successfully read resource will be returned as the &#x60;paging.next.after&#x60; JSON property of a paged response containing more results.
        # @option opts [Boolean] :archived Whether to include archived custom actions.
        # @return [Array<(CollectionResponseExtensionActionDefinitionForwardPaging, Integer, Hash)>] CollectionResponseExtensionActionDefinitionForwardPaging data, response status code and response headers
        def get_page_with_http_info(app_id, opts = {})
          if @api_client.config.debugging
            @api_client.config.logger.debug 'Calling API: DefinitionsApi.get_page ...'
          end
          # verify the required parameter 'app_id' is set
          if @api_client.config.client_side_validation && app_id.nil?
            fail ArgumentError, "Missing the required parameter 'app_id' when calling DefinitionsApi.get_page"
          end
          # resource path
          local_var_path = '/automation/v4/actions/{appId}'.sub('{' + 'appId' + '}', CGI.escape(app_id.to_s))

          # query parameters
          query_params = opts[:query_params] || {}
          query_params[:'limit'] = opts[:'limit'] if !opts[:'limit'].nil?
          query_params[:'after'] = opts[:'after'] if !opts[:'after'].nil?
          query_params[:'archived'] = opts[:'archived'] if !opts[:'archived'].nil?

          # header parameters
          header_params = opts[:header_params] || {}
          # HTTP header 'Accept' (if needed)
          header_params['Accept'] = @api_client.select_header_accept(['application/json', '*/*'])

          # form parameters
          form_params = opts[:form_params] || {}

          # http body (model)
          post_body = opts[:body] 

          # return_type
          return_type = opts[:return_type] || 'CollectionResponseExtensionActionDefinitionForwardPaging' 

          # auth_names
          auth_names = opts[:auth_names] || ['developer_hapikey']

          new_options = opts.merge(
            :header_params => header_params,
            :query_params => query_params,
            :form_params => form_params,
            :body => post_body,
            :auth_names => auth_names,
            :return_type => return_type
          )

          data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
          if @api_client.config.debugging
            @api_client.config.logger.debug "API called: DefinitionsApi#get_page\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
          end
          return data, status_code, headers
        end

        # Update a custom action
        # Updates a custom workflow action with new values for the specified fields.
        # @param definition_id [String] The ID of the custom workflow action.
        # @param app_id [Integer] 
        # @param extension_action_definition_patch [ExtensionActionDefinitionPatch] The custom workflow action fields to be updated.
        # @param [Hash] opts the optional parameters
        # @return [ExtensionActionDefinition]
        def update(definition_id, app_id, extension_action_definition_patch, opts = {})
          data, _status_code, _headers = update_with_http_info(definition_id, app_id, extension_action_definition_patch, opts)
          data
        end

        # Update a custom action
        # Updates a custom workflow action with new values for the specified fields.
        # @param definition_id [String] The ID of the custom workflow action.
        # @param app_id [Integer] 
        # @param extension_action_definition_patch [ExtensionActionDefinitionPatch] The custom workflow action fields to be updated.
        # @param [Hash] opts the optional parameters
        # @return [Array<(ExtensionActionDefinition, Integer, Hash)>] ExtensionActionDefinition data, response status code and response headers
        def update_with_http_info(definition_id, app_id, extension_action_definition_patch, opts = {})
          if @api_client.config.debugging
            @api_client.config.logger.debug 'Calling API: DefinitionsApi.update ...'
          end
          # verify the required parameter 'definition_id' is set
          if @api_client.config.client_side_validation && definition_id.nil?
            fail ArgumentError, "Missing the required parameter 'definition_id' when calling DefinitionsApi.update"
          end
          # verify the required parameter 'app_id' is set
          if @api_client.config.client_side_validation && app_id.nil?
            fail ArgumentError, "Missing the required parameter 'app_id' when calling DefinitionsApi.update"
          end
          # verify the required parameter 'extension_action_definition_patch' is set
          if @api_client.config.client_side_validation && extension_action_definition_patch.nil?
            fail ArgumentError, "Missing the required parameter 'extension_action_definition_patch' when calling DefinitionsApi.update"
          end
          # resource path
          local_var_path = '/automation/v4/actions/{appId}/{definitionId}'.sub('{' + 'definitionId' + '}', CGI.escape(definition_id.to_s)).sub('{' + 'appId' + '}', CGI.escape(app_id.to_s))

          # query parameters
          query_params = opts[:query_params] || {}

          # header parameters
          header_params = opts[:header_params] || {}
          # HTTP header 'Accept' (if needed)
          header_params['Accept'] = @api_client.select_header_accept(['application/json', '*/*'])
          # HTTP header 'Content-Type'
          header_params['Content-Type'] = @api_client.select_header_content_type(['application/json'])

          # form parameters
          form_params = opts[:form_params] || {}

          # http body (model)
          post_body = opts[:body] || @api_client.object_to_http_body(extension_action_definition_patch) 

          # return_type
          return_type = opts[:return_type] || 'ExtensionActionDefinition' 

          # auth_names
          auth_names = opts[:auth_names] || ['developer_hapikey']

          new_options = opts.merge(
            :header_params => header_params,
            :query_params => query_params,
            :form_params => form_params,
            :body => post_body,
            :auth_names => auth_names,
            :return_type => return_type
          )

          data, status_code, headers = @api_client.call_api(:PATCH, local_var_path, new_options)
          if @api_client.config.debugging
            @api_client.config.logger.debug "API called: DefinitionsApi#update\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
          end
          return data, status_code, headers
        end
      end
    end
  end
end
