=begin
#Accounting Extension

#These APIs allow you to interact with HubSpot's Accounting Extension. It allows you to: * Specify the URLs that HubSpot will use when making webhook requests to your external accounting system. * Respond to webhook calls made to your external accounting system by HubSpot 

The version of the OpenAPI document: v3

Generated by: https://openapi-generator.tech
OpenAPI Generator version: 4.3.1

=end

require 'cgi'

module Hubspot
  module Crm
    module Extensions
      module Accounting
        class SyncApi
          attr_accessor :api_client

          def initialize(api_client = ApiClient.default)
            @api_client = api_client
          end
          # Import contacts
          # Imports contacts' properties from an external accounting system to HubSpot. Import details, including property mappings, must be configured previously in HubSpot infrastructure.
          # @param app_id [Integer] The ID of the accounting app. This is the identifier of the application created in your HubSpot developer portal.
          # @param sync_contacts_request [SyncContactsRequest] 
          # @param [Hash] opts the optional parameters
          # @return [ActionResponse]
          def create_contact(app_id, sync_contacts_request, opts = {})
            data, _status_code, _headers = create_contact_with_http_info(app_id, sync_contacts_request, opts)
            data
          end

          # Import contacts
          # Imports contacts&#39; properties from an external accounting system to HubSpot. Import details, including property mappings, must be configured previously in HubSpot infrastructure.
          # @param app_id [Integer] The ID of the accounting app. This is the identifier of the application created in your HubSpot developer portal.
          # @param sync_contacts_request [SyncContactsRequest] 
          # @param [Hash] opts the optional parameters
          # @return [Array<(ActionResponse, Integer, Hash)>] ActionResponse data, response status code and response headers
          def create_contact_with_http_info(app_id, sync_contacts_request, opts = {})
            if @api_client.config.debugging
              @api_client.config.logger.debug 'Calling API: SyncApi.create_contact ...'
            end
            # verify the required parameter 'app_id' is set
            if @api_client.config.client_side_validation && app_id.nil?
              fail ArgumentError, "Missing the required parameter 'app_id' when calling SyncApi.create_contact"
            end
            # verify the required parameter 'sync_contacts_request' is set
            if @api_client.config.client_side_validation && sync_contacts_request.nil?
              fail ArgumentError, "Missing the required parameter 'sync_contacts_request' when calling SyncApi.create_contact"
            end
            # resource path
            local_var_path = '/crm/v3/extensions/accounting/sync/{appId}/contacts'.sub('{' + 'appId' + '}', CGI.escape(app_id.to_s))

            # query parameters
            query_params = opts[:query_params] || {}

            # header parameters
            header_params = opts[:header_params] || {}
            # HTTP header 'Accept' (if needed)
            header_params['Accept'] = @api_client.select_header_accept(['application/json', '*/*'])
            # HTTP header 'Content-Type'
            header_params['Content-Type'] = @api_client.select_header_content_type(['application/json'])

            # form parameters
            form_params = opts[:form_params] || {}

            # http body (model)
            post_body = opts[:body] || @api_client.object_to_http_body(sync_contacts_request) 

            # return_type
            return_type = opts[:return_type] || 'ActionResponse' 

            # auth_names
            auth_names = opts[:auth_names] || ['hapikey', 'oauth2']

            new_options = opts.merge(
              :header_params => header_params,
              :query_params => query_params,
              :form_params => form_params,
              :body => post_body,
              :auth_names => auth_names,
              :return_type => return_type
            )

            data, status_code, headers = @api_client.call_api(:POST, local_var_path, new_options)
            if @api_client.config.debugging
              @api_client.config.logger.debug "API called: SyncApi#create_contact\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
            end
            return data, status_code, headers
          end

          # Import products
          # Imports products' properties from an external accounting system to HubSpot. Import details, including property mappings, must be configured previously in HubSpot infrastructure.
          # @param app_id [Integer] The ID of the accounting app. This is the identifier of the application created in your HubSpot developer portal.
          # @param sync_products_request [SyncProductsRequest] 
          # @param [Hash] opts the optional parameters
          # @return [ActionResponse]
          def create_product(app_id, sync_products_request, opts = {})
            data, _status_code, _headers = create_product_with_http_info(app_id, sync_products_request, opts)
            data
          end

          # Import products
          # Imports products&#39; properties from an external accounting system to HubSpot. Import details, including property mappings, must be configured previously in HubSpot infrastructure.
          # @param app_id [Integer] The ID of the accounting app. This is the identifier of the application created in your HubSpot developer portal.
          # @param sync_products_request [SyncProductsRequest] 
          # @param [Hash] opts the optional parameters
          # @return [Array<(ActionResponse, Integer, Hash)>] ActionResponse data, response status code and response headers
          def create_product_with_http_info(app_id, sync_products_request, opts = {})
            if @api_client.config.debugging
              @api_client.config.logger.debug 'Calling API: SyncApi.create_product ...'
            end
            # verify the required parameter 'app_id' is set
            if @api_client.config.client_side_validation && app_id.nil?
              fail ArgumentError, "Missing the required parameter 'app_id' when calling SyncApi.create_product"
            end
            # verify the required parameter 'sync_products_request' is set
            if @api_client.config.client_side_validation && sync_products_request.nil?
              fail ArgumentError, "Missing the required parameter 'sync_products_request' when calling SyncApi.create_product"
            end
            # resource path
            local_var_path = '/crm/v3/extensions/accounting/sync/{appId}/products'.sub('{' + 'appId' + '}', CGI.escape(app_id.to_s))

            # query parameters
            query_params = opts[:query_params] || {}

            # header parameters
            header_params = opts[:header_params] || {}
            # HTTP header 'Accept' (if needed)
            header_params['Accept'] = @api_client.select_header_accept(['application/json', '*/*'])
            # HTTP header 'Content-Type'
            header_params['Content-Type'] = @api_client.select_header_content_type(['application/json'])

            # form parameters
            form_params = opts[:form_params] || {}

            # http body (model)
            post_body = opts[:body] || @api_client.object_to_http_body(sync_products_request) 

            # return_type
            return_type = opts[:return_type] || 'ActionResponse' 

            # auth_names
            auth_names = opts[:auth_names] || ['hapikey', 'oauth2']

            new_options = opts.merge(
              :header_params => header_params,
              :query_params => query_params,
              :form_params => form_params,
              :body => post_body,
              :auth_names => auth_names,
              :return_type => return_type
            )

            data, status_code, headers = @api_client.call_api(:POST, local_var_path, new_options)
            if @api_client.config.debugging
              @api_client.config.logger.debug "API called: SyncApi#create_product\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
            end
            return data, status_code, headers
          end
        end
      end
    end
  end
end
